(in-package :om)

(defclass screamerboxes (OMBoxCall) ()
   (:documentation "Screamer boxes"))

(defmethod omNG-box-value ((self screamerboxes) &optional (numout 0))
   "Eval the output indexed by 'numout' for the box 'self'. In this method we call the generic function reference of 'self'."
   (handler-bind ((error #'(lambda (c)
                             (when *msg-error-label-on*
                               (om-message-dialog (string+ "Error while evaluating the box " (string (name self)) " " 
                                                                               (om-report-condition c)
                                                                               )
                                               :size (om-make-point 300 200))
                               (om-abort)))))
     (cond
      ((or (equal (allow-lock self) "l") 
           (equal (allow-lock self) "o")  
           (and (equal (allow-lock self) "x") (value self)) 
           (and (equal (allow-lock self) "&") (ev-once-p self))) (call-next-method))
      (t (let* ((args  (loop for input in (inputs self)
                             when (not (keyword-input-p input)) collect (omNG-box-value  input)))
                (qargs (loop for val in args collect (if (or (symbolp val) (omlistp val)) `',val val))) 
                (themethod (compute-applicable-methods (fdefinition (reference self)) args)) rep)
           (if (null themethod)
             (progn (dialog-message (string+ "no method is defined for inputs in box " (name self)))
                    (abort))
             (progn
               (when (and (EditorFrame (car themethod)) (not (compiled? (car themethod))))
                 (modify-genfun (EditorFrame (car themethod))))
               (setf rep 
                     (case *screamer-valuation*
                       (0 (multiple-value-list (eval `(s::one-value (,(intern (string (reference self)) :s) ,.qargs)))))
                       (1 (multiple-value-list (eval `(s::all-values (,(intern (string (reference self)) :s) ,.qargs)))))  
                       (2 (multiple-value-list (eval `(s::print-values (,(intern (string (reference self)) :s) ,.qargs))))))))
             )
           (when (equal (allow-lock self) "&")
             (setf (ev-once-p self) t)
             (setf (value self) rep))
           (when (equal (allow-lock self) "x")
             (setf (value self) rep))
           (nth numout rep)))))) 


(defmacro evalsc (fun arg)
  `(let ((varg (loop for item in ,arg collect (quote item))))
     (print (,fun varg))))


(defmethod special-lambda-value ((self screamerboxes) symbol)
   "Eval a box in lambda mode."
   (multiple-value-bind (nesymbs args) (get-args-eval-currry self)
     (eval `#'(lambda ,(reverse nesymbs)
                (case *screamer-valuation*
                  (0 (s::one-value (,symbol ,.args)))
                  (1 (s::all-values (,symbol ,.args)))  
                  (2 (s::print-values (,symbol ,.args))))))))

(defmethod gen-code-call ((self screamerboxes) &optional args)
   (let ((screamerfun `,(intern (string (reference self)) :s)))
     
     `(,screamerfun ,.(decode self))))


(defmethod gen-code ((self screamerboxes) numout)
   "Generate Lisp code for the box 'self'."
   (let ((screamerfun `,(intern (string (reference self)) :s)))
     (cond
      ((equal (allow-lock self) "&") 
       (gen-code-for-ev-once self numout))
      ((equal (allow-lock self) "x")
       `(nth ,numout ,(gen-code (value self) 0)))
      ((equal (allow-lock self) "o") 
       `',(reference self))
      ((equal (allow-lock self) "l") 
       (curry-lambda-code self screamerfun))
      (t  `(,screamerfun ,.(decode self))))))


;Faire la meme chose pour les autres fontions



